﻿using System;
using System.Globalization;
using System.IO;
using System.Net;
using System.Windows.Forms;
using System.Xml;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace RasterRotation
{
    public partial class MainForm : Form
    {
        private RotationProjection projection;
        private string selectLayer;

        public MainForm()
        {
            InitializeComponent();
        }

        private void WindowLoaded(object sender, EventArgs e)
        {
            selectLayer = "OSM Layer";
            winformsMap.MapUnit = GeographyUnit.Meter;
            winformsMap.CurrentExtent = new RectangleShape(-20037508.352, 20037508.352, 20037508.352, -20037508.352);
            projection = new RotationProjection(0.0);
            projection.Open();

            InitializeLayers();

            winformsMap.Refresh();
        }

        private void InitializeLayers()
        {
            LayerOverlay rasterLayerOverlay = new LayerOverlay();
            rasterLayerOverlay.Layers.Add("OSM Layer", new OpenStreetMapLayer());

            BingMapsLayer bingMapsLayer = new BingMapsLayer();
            bingMapsLayer.MapType = BingMapsMapType.Aerial;
            bingMapsLayer.IsVisible = false;
            rasterLayerOverlay.Layers.Add("Bing Maps Layer", bingMapsLayer);

            GoogleMapsLayer googleMapsLayer = new GoogleMapsLayer();
            googleMapsLayer.MapType = GoogleMapsMapType.Satellite;
            googleMapsLayer.TileMode = GoogleMapsTileMode.SingleTile;
            googleMapsLayer.WebProxy = new WebProxy("127.0.0.1:1080");
            googleMapsLayer.IsVisible = false;
            rasterLayerOverlay.Layers.Add("Google Maps Layer", googleMapsLayer);

            Jpeg2000RasterLayer jpeg2000RasterLayer = new Jpeg2000RasterLayer(@"..\..\App_Data\Jpeg2000\World.jp2", @"..\..\App_Data\Jpeg2000\World.j2w");
            jpeg2000RasterLayer.IsVisible = false;
            rasterLayerOverlay.Layers.Add("Jpeg2000 Layer", jpeg2000RasterLayer);

            GeoTiffRasterLayer geoTiffRasterLayer = new GeoTiffRasterLayer(@"..\..\App_Data\Tiff\World.tif", @"..\..\App_Data\Tiff\World.tfw");
            geoTiffRasterLayer.IsVisible = false;
            rasterLayerOverlay.Layers.Add("GeoTiff Layer", geoTiffRasterLayer);

            GdiPlusRasterLayer gdiPlusRasterLayer = new GdiPlusRasterLayer(@"..\..\App_Data\PNG\World.png", new RectangleShape(-20037508.352, 20037508.352, 20037508.352, -20037508.352));
            gdiPlusRasterLayer.IsVisible = false;
            rasterLayerOverlay.Layers.Add("GdiPlus Layer", gdiPlusRasterLayer);

            MrSidRasterLayer mrSidRasterLayer = new MrSidRasterLayer(@"..\..\App_Data\MrSID\world.sid");
            mrSidRasterLayer.IsVisible = false;
            rasterLayerOverlay.Layers.Add("MrSID Layer", mrSidRasterLayer);

            winformsMap.Overlays.Add("RasterLayerOverlay", rasterLayerOverlay);
        }

        private void SelectedLayerChanged(object sender, EventArgs e)
        {
            RadioButton radioButton = (RadioButton)sender;
            if (!radioButton.Checked) return;

            RefreshLayer(radioButton.Text);
        }

        private void RefreshLayer(string layerType)
        {
            LayerOverlay layerOverlay = (LayerOverlay)winformsMap.Overlays["RasterLayerOverlay"];
            foreach (var layer in layerOverlay.Layers)
            {
                layer.IsVisible = false;
            }

            Layer currentLayer = layerOverlay.Layers[layerType];
            currentLayer.IsVisible = true;

            selectLayer = layerType;
            switch (layerType)
            {
                case "OSM Layer":
                    OpenStreetMapLayer openStreetMapLayer = (OpenStreetMapLayer)currentLayer;
                    openStreetMapLayer.ProjectionFromSphericalMercator = projection;
                    break;

                case "Bing Maps Layer":
                    BingMapsLayer bingMapsLayer = (BingMapsLayer)currentLayer;
                    BingMapsApplicationIdPromptForm inputBingMapKeyWindow = new BingMapsApplicationIdPromptForm();
                    inputBingMapKeyWindow.StartPosition = FormStartPosition.CenterParent;
                    if (inputBingMapKeyWindow.ShowDialog() == DialogResult.OK && IsBingMapsKeyValid(inputBingMapKeyWindow.ApplicationId, bingMapsLayer.MapType))
                    {
                        bingMapsLayer.ApplicationId = inputBingMapKeyWindow.ApplicationId;
                        bingMapsLayer.ProjectionFromSphericalMercator = projection;
                    }
                    break;

                case "Google Maps Layer":
                    GoogleMapsLayer googleMapsLayer = (GoogleMapsLayer)currentLayer;
                    googleMapsLayer.ProjectionFromSphericalMercator = projection;
                    break;

                case "Jpeg2000 Layer":
                case "GeoTiff Layer":
                case "GdiPlus Layer":
                case "MrSID Layer":
                    RasterLayer rasterLayer = (RasterLayer)currentLayer;
                    rasterLayer.ImageSource.Projection = projection;
                    break;
            }

            winformsMap.CurrentExtent = projection.GetUpdatedExtent(winformsMap.CurrentExtent);
            winformsMap.Refresh();
        }

        private void btnRotateLeft_Click(object sender, EventArgs e)
        {
            projection.Angle += 10;
            RefreshLayer(selectLayer);
        }

        private void btnRotateRight_Click(object sender, EventArgs e)
        {
            projection.Angle -= 10;
            RefreshLayer(selectLayer);
        }

        private static bool IsBingMapsKeyValid(string bingMapsKey, BingMapsMapType mapType)
        {
            bool isValid = false;
            string bingLoginServiceTemplate = "http://dev.virtualearth.net/REST/v1/Imagery/Metadata/{0}?&incl=ImageryProviders&o=xml&key={1}";
            try
            {
                string loginServiceUri = String.Format(CultureInfo.InvariantCulture
                              , bingLoginServiceTemplate, mapType, bingMapsKey);

                WebRequest request = WebRequest.Create(loginServiceUri);
                WebResponse response = request.GetResponse();
                Stream stream = response.GetResponseStream();
                XmlDocument xDoc = new XmlDocument();
                xDoc.Load(stream);
                XmlNamespaceManager xmlNamespaceManager = new XmlNamespaceManager(xDoc.NameTable);
                xmlNamespaceManager.AddNamespace("bing", "http://schemas.microsoft.com/search/local/ws/rest/v1");

                XmlNode root = xDoc.SelectSingleNode("bing:Response", xmlNamespaceManager);
                XmlNode imageUrlElement = root.SelectSingleNode("bing:ResourceSets/bing:ResourceSet/bing:Resources/bing:ImageryMetadata/bing:ImageUrl", xmlNamespaceManager);
                XmlNodeList subdomainsElement
                    = root.SelectNodes("bing:ResourceSets/bing:ResourceSet/bing:Resources/bing:ImageryMetadata/bing:ImageUrlSubdomains/bing:string", xmlNamespaceManager);

                if (imageUrlElement != null && subdomainsElement != null)
                {
                    isValid = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

            return isValid;
        }
    }
}
